@@ -65,6 +65,7 @@ static const gchar *ide_application_credits_authors[] = {
   "Debarshi Dutta",
   "Dimitrios Christidis",
   "Dimitris Zenios",
+  "Dor Askayo",
   "Dušan Kazik",
   "Ekaterina Gerasimova",
   "Elad Alfassa",
@@ -78,6 +79,7 @@ static const gchar *ide_application_credits_authors[] = {
   "Gábor Kelemen",
   "Garrett Regier",
   "Gautier Pelloux-Prayer",
+  "Gennady Kovalev",
   "Georg Vienna",
   "Giovanni Campagna",
   "Hashem Nasarat",
@@ -117,6 +119,7 @@ static const gchar *ide_application_credits_authors[] = {
   "namanyadav12",
   "Paolo Borelli",
   "Patrick Griffis",
+  "Peter Sonntag",
   "Pedro Albuquerque",
   "Pete Travis",
   "Philip Withnall",
@@ -134,6 +137,7 @@ static const gchar *ide_application_credits_authors[] = {
   "Tobias Schönberg",
   "Tom Tryfonidis",
   "Trinh Anh Ngoc",
+  "Umang Jain",
   "Wolf Vollprecht",
   "Yannick Inizan",
   "Yosef Or Boczko",
