@@ -19,6 +19,7 @@
 #define G_LOG_DOMAIN "ide-unsaved-files"
 
 #include <errno.h>
+#include <fcntl.h>
 #include <glib/gstdio.h>
 #include <string.h>
 
@@ -30,6 +31,7 @@
 #include "buffers/ide-unsaved-file.h"
 #include "buffers/ide-unsaved-files.h"
 #include "projects/ide-project.h"
+#include "util/ide-directory-reaper.h"
 
 typedef struct
 {
@@ -154,6 +156,15 @@ hash_uri (const gchar *uri)
   return ret;
 }
 
+static gchar *
+get_buffers_dir (void)
+{
+  return g_build_filename (g_get_user_cache_dir (),
+                           "gnome-builder",
+                           "buffers",
+                           NULL);
+}
+
 static void
 ide_unsaved_files_save_worker (GTask        *task,
                                gpointer      source_object,
@@ -485,23 +496,48 @@ ide_unsaved_files_remove (IdeUnsavedFiles *self,
 static void
 setup_tempfile (GFile  *file,
                 gint   *temp_fd,
-                gchar **temp_path)
+                gchar **temp_path_out)
 {
+  g_autofree gchar *tmpdir = NULL;
   g_autofree gchar *name = NULL;
-  g_autofree gchar *template = NULL;
+  g_autofree gchar *shortname = NULL;
+  g_autofree gchar *tmpl_path = NULL;
   const gchar *suffix;
 
   g_assert (G_IS_FILE (file));
   g_assert (temp_fd);
-  g_assert (temp_path);
+  g_assert (temp_path_out);
 
   *temp_fd = -1;
-  *temp_path = NULL;
+  *temp_path_out = NULL;
 
+  /* Get the suffix for the filename so that we can add it as the suffix to
+   * our temporary file. That increases the chance that anything sniffing
+   * content-type will work correctly.
+   */
   name = g_file_get_basename (file);
   suffix = strrchr (name, '.') ?: "";
-  template = g_strdup_printf ("builder_codeassistant_XXXXXX%s", suffix);
-  *temp_fd = g_file_open_tmp (template, temp_path, NULL);
+
+
+  /*
+   * We want to create our tempfile within a custom directory. It turns out
+   * that g_mkstemp_full() does not do directory checks in the template, so
+   * we can pass our own directory to be used instead of $TMPDIR. We need to
+   * control the directory so that we can ensure we have one that is available
+   * to both the flatpak runtime and the host system.
+   */
+  tmpdir = get_buffers_dir ();
+  shortname = g_strdup_printf ("buffer-XXXXXX%s", suffix);
+  tmpl_path = g_build_filename (tmpdir, shortname, NULL);
+
+  /* Ensure the directory exists */
+  if (!g_file_test (tmpdir, G_FILE_TEST_IS_DIR))
+    g_mkdir_with_parents (tmpdir, 0750);
+
+  /* Now try to open our custom tempfile in the directory we control. */
+  *temp_fd = g_mkstemp_full (tmpl_path, O_RDWR, 0664);
+  if (*temp_fd != -1)
+    *temp_path_out = g_steal_pointer (&tmpl_path);
 }
 
 void
@@ -685,6 +721,34 @@ ide_unsaved_files_get_sequence (IdeUnsavedFiles *self)
   return priv->sequence;
 }
 
+static void
+ide_unsaved_files_set_context (IdeObject  *object,
+                               IdeContext *context)
+{
+  IdeUnsavedFiles *self = (IdeUnsavedFiles *)object;
+  g_autoptr(IdeDirectoryReaper) reaper = NULL;
+  g_autoptr(GFile) buffersdir = NULL;
+  g_autofree gchar *path = NULL;
+
+  g_assert (IDE_IS_UNSAVED_FILES (self));
+  g_assert (!context || IDE_IS_CONTEXT (context));
+
+  IDE_OBJECT_CLASS (ide_unsaved_files_parent_class)->set_context (object, context);
+
+  reaper = ide_directory_reaper_new ();
+
+  /*
+   * Setup a reaper to cleanup old files in case that we left some around
+   * after a previous crash.
+   */
+  path = get_buffers_dir ();
+  buffersdir = g_file_new_for_path (path);
+  ide_directory_reaper_add_directory (reaper, buffersdir, G_TIME_SPAN_DAY);
+
+  /* Now cleanup the old files */
+  ide_directory_reaper_execute_async (reaper, NULL, NULL, NULL);
+}
+
 static void
 ide_unsaved_files_finalize (GObject *object)
 {
@@ -700,8 +764,11 @@ static void
 ide_unsaved_files_class_init (IdeUnsavedFilesClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  IdeObjectClass *ide_object_class = IDE_OBJECT_CLASS (klass);
 
   object_class->finalize = ide_unsaved_files_finalize;
+
+  ide_object_class->set_context = ide_unsaved_files_set_context;
 }
 
 static void
