@@ -201,6 +201,13 @@ struct _IdeBuildPipeline
    * If we are in the middle of a clean operation.
    */
   guint in_clean : 1;
+
+  /*
+   * Precalculation if we need to look for errors on stdout. We can't rely
+   * on @current_stage for this, becase log entries might come in
+   * asynchronously and after the processes/stage has completed.
+   */
+  guint errors_on_stdout : 1;
 };
 
 typedef enum
@@ -417,21 +424,31 @@ create_diagnostic (IdeBuildPipeline *self,
 
   parsed.severity = parse_severity (level);
 
-  if (!g_path_is_absolute (filename) && self->errfmt_current_dir != NULL)
+  if (!g_path_is_absolute (filename))
     {
-      const gchar *basedir = self->errfmt_current_dir;
       gchar *path;
 
-      if (g_str_has_prefix (basedir, self->errfmt_top_dir))
+      if (self->errfmt_current_dir != NULL)
         {
-          basedir += strlen (self->errfmt_top_dir);
-          if (*basedir == '/')
-            basedir++;
-        }
+          const gchar *basedir = self->errfmt_current_dir;
 
-      path = g_build_filename (basedir, filename, NULL);
-      g_free (filename);
-      filename = path;
+          if (g_str_has_prefix (basedir, self->errfmt_top_dir))
+            {
+              basedir += strlen (self->errfmt_top_dir);
+              if (*basedir == G_DIR_SEPARATOR)
+                basedir++;
+            }
+
+          path = g_build_filename (basedir, filename, NULL);
+          g_free (filename);
+          filename = path;
+        }
+      else
+        {
+          path = g_build_filename (self->builddir, filename, NULL);
+          g_free (filename);
+          filename = path;
+        }
     }
 
   context = ide_object_get_context (IDE_OBJECT (self));
@@ -466,15 +483,15 @@ ide_build_pipeline_log_observer (IdeBuildLogStream  stream,
                                  gpointer           user_data)
 {
   IdeBuildPipeline *self = user_data;
-  const gchar *enterdir;
   g_autofree gchar *filtered_message = NULL;
+  const gchar *enterdir;
 
   g_assert (stream == IDE_BUILD_LOG_STDOUT || stream == IDE_BUILD_LOG_STDERR);
   g_assert (IDE_IS_BUILD_PIPELINE (self));
   g_assert (message != NULL);
 
 #define ENTERING_DIRECTORY_BEGIN "Entering directory '"
-#define ENTERING_DIRECTORY_END   "'\n"
+#define ENTERING_DIRECTORY_END   "'"
 
   if (message_len < 0)
     message_len = strlen (message);
@@ -484,43 +501,63 @@ ide_build_pipeline_log_observer (IdeBuildLogStream  stream,
 
   filtered_message = ide_build_utils_color_codes_filtering (message);
 
-  /*
-   * This expects LANG=C, which is defined in the autotools Builder.
-   * Not the most ideal decoupling of logic, but we don't have a whole
-   * lot to work with here.
-   */
-  if (NULL != (enterdir = strstr (filtered_message, ENTERING_DIRECTORY_BEGIN)) &&
-      g_str_has_suffix (enterdir, ENTERING_DIRECTORY_END))
+  if (stream == IDE_BUILD_LOG_STDOUT)
     {
-      gssize len;
-
-      enterdir += IDE_LITERAL_LENGTH (ENTERING_DIRECTORY_BEGIN);
-      len = strlen (enterdir) - IDE_LITERAL_LENGTH (ENTERING_DIRECTORY_END);
-
-      if (len > 0)
+      /*
+       * This expects LANG=C, which is defined in the autotools Builder.
+       * Not the most ideal decoupling of logic, but we don't have a whole
+       * lot to work with here.
+       */
+      if (NULL != (enterdir = strstr (filtered_message, ENTERING_DIRECTORY_BEGIN)) &&
+          g_str_has_suffix (enterdir, ENTERING_DIRECTORY_END))
         {
-          g_free (self->errfmt_current_dir);
-          self->errfmt_current_dir = g_strndup (enterdir, len);
-          if (self->errfmt_top_dir == NULL)
-            self->errfmt_top_dir = g_strndup (enterdir, len);
-        }
+          gssize len;
 
-      return;
+          enterdir += IDE_LITERAL_LENGTH (ENTERING_DIRECTORY_BEGIN);
+
+          /* Translate to relative paths for out-of-tree builds */
+          if (g_str_has_prefix (enterdir, self->builddir))
+            {
+              enterdir += strlen (self->builddir);
+              if (*enterdir == G_DIR_SEPARATOR)
+                enterdir++;
+            }
+
+          len = strlen (enterdir) - IDE_LITERAL_LENGTH (ENTERING_DIRECTORY_END);
+
+          if (len > 0)
+            {
+              g_free (self->errfmt_current_dir);
+              self->errfmt_current_dir = g_strndup (enterdir, len);
+              if (self->errfmt_top_dir == NULL)
+                self->errfmt_top_dir = g_strndup (enterdir, len);
+            }
+
+          return;
+        }
     }
 
-  for (guint i = 0; i < self->errfmts->len; i++)
+  /*
+   * Unfortunately, some build engines such as Ninja refuse to pass errors on
+   * stderr like the tooling they abstract. So we must parse stdout in addition
+   * to stderr to extract errors.
+   */
+  if (stream == IDE_BUILD_LOG_STDERR || self->errors_on_stdout)
     {
-      const ErrorFormat *errfmt = &g_array_index (self->errfmts, ErrorFormat, i);
-      g_autoptr(GMatchInfo) match_info = NULL;
-
-      if (g_regex_match (errfmt->regex, filtered_message, 0, &match_info))
+      for (guint i = 0; i < self->errfmts->len; i++)
         {
-          g_autoptr(IdeDiagnostic) diagnostic = create_diagnostic (self, match_info);
+          const ErrorFormat *errfmt = &g_array_index (self->errfmts, ErrorFormat, i);
+          g_autoptr(GMatchInfo) match_info = NULL;
 
-          if (diagnostic != NULL)
+          if (g_regex_match (errfmt->regex, filtered_message, 0, &match_info))
             {
-              ide_build_pipeline_emit_diagnostic (self, diagnostic);
-              return;
+              g_autoptr(IdeDiagnostic) diagnostic = create_diagnostic (self, match_info);
+
+              if (diagnostic != NULL)
+                {
+                  ide_build_pipeline_emit_diagnostic (self, diagnostic);
+                  return;
+                }
             }
         }
     }
@@ -650,6 +687,19 @@ ide_build_pipeline_real_started (IdeBuildPipeline *self)
 
   g_assert (IDE_IS_BUILD_PIPELINE (self));
 
+  self->errors_on_stdout = FALSE;
+
+  for (guint i = 0; i < self->pipeline->len; i++)
+    {
+      PipelineEntry *entry = &g_array_index (self->pipeline, PipelineEntry, i);
+
+      if (ide_build_stage_get_check_stdout (entry->stage))
+        {
+          self->errors_on_stdout = TRUE;
+          break;
+        }
+    }
+
   IDE_EXIT;
 }
 
@@ -1924,6 +1974,8 @@ ide_build_pipeline_create_launcher (IdeBuildPipeline  *self,
 
       ide_subprocess_launcher_set_clear_env (ret, TRUE);
       ide_subprocess_launcher_overlay_environment (ret, env);
+      /* Always ignore V=1 from configurations */
+      ide_subprocess_launcher_setenv (ret, "V", "0", TRUE);
       ide_subprocess_launcher_set_cwd (ret, ide_build_pipeline_get_builddir (self));
       ide_subprocess_launcher_set_flags (ret,
                                          (G_SUBPROCESS_FLAGS_STDERR_PIPE |
