@@ -133,6 +133,17 @@ sort_priority (gconstpointer a,
 {
   IdeBuildSystem **as = (IdeBuildSystem **)a;
   IdeBuildSystem **bs = (IdeBuildSystem **)b;
+  g_autofree gchar *id_a = ide_build_system_get_id (*as);
+  g_autofree gchar *id_b = ide_build_system_get_id (*bs);
+  const gchar *build_system_hint = data;
+
+  if (build_system_hint != NULL)
+    {
+      if (g_strcmp0 (build_system_hint, id_a) == 0)
+        return -1;
+      else if (g_strcmp0 (build_system_hint, id_b) == 0)
+        return 1;
+    }
 
   return ide_build_system_get_priority (*as) - ide_build_system_get_priority (*bs);
 }
@@ -141,6 +152,7 @@ sort_priority (gconstpointer a,
  * ide_build_system_new_async:
  * @context: #IdeBuildSystem
  * @project_file: A #GFile containing the directory or project file.
+ * @build_system_hint: A hint for the build system to use
  * @cancellable: (allow-none): A #GCancellable
  * @callback: A callback to execute upon completion
  * @user_data: User data for @callback.
@@ -155,6 +167,7 @@ sort_priority (gconstpointer a,
 void
 ide_build_system_new_async (IdeContext          *context,
                             GFile               *project_file,
+                            const gchar         *build_system_hint,
                             GCancellable        *cancellable,
                             GAsyncReadyCallback  callback,
                             gpointer             user_data)
@@ -163,8 +176,10 @@ ide_build_system_new_async (IdeContext          *context,
   g_return_if_fail (G_IS_FILE (project_file));
   g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
 
+  IDE_TRACE_MSG ("Creating build system with hint \"%s\"", build_system_hint ?: "");
+
   ide_object_new_for_extension_async (IDE_TYPE_BUILD_SYSTEM,
-                                      sort_priority, NULL,
+                                      sort_priority, (gpointer)build_system_hint,
                                       G_PRIORITY_DEFAULT,
                                       cancellable,
                                       callback,
@@ -306,7 +321,7 @@ ide_build_system_get_builddir (IdeBuildSystem   *self,
       runtime_id = ide_configuration_get_runtime_id (configuration);
 
       name = g_strdelimit (g_strdup_printf ("%s-%s-%s", config_id, device_id, runtime_id),
-                           G_DIR_SEPARATOR_S, '-');
+                           "@:/", '-');
 
       ret = g_build_filename (g_get_user_cache_dir (),
                               "gnome-builder",
@@ -318,3 +333,14 @@ ide_build_system_get_builddir (IdeBuildSystem   *self,
 
   IDE_RETURN (ret);
 }
+
+gchar *
+ide_build_system_get_id (IdeBuildSystem *self)
+{
+  g_return_val_if_fail (IDE_IS_BUILD_SYSTEM (self), NULL);
+
+  if (IDE_BUILD_SYSTEM_GET_IFACE (self)->get_id)
+    return IDE_BUILD_SYSTEM_GET_IFACE (self)->get_id (self);
+
+  return g_strdup (G_OBJECT_TYPE_NAME (self));
+}
