@@ -193,7 +193,7 @@ ide_editor_frame_update_ruler (IdeEditorFrame *self)
 
 static void
 ide_editor_frame_set_show_ruler (IdeEditorFrame *self,
-                                gboolean       show_ruler)
+                                 gboolean        show_ruler)
 {
   g_assert (IDE_IS_EDITOR_FRAME (self));
 
@@ -207,7 +207,7 @@ ide_editor_frame_set_show_ruler (IdeEditorFrame *self,
 
 static void
 ide_editor_frame_animate_map (IdeEditorFrame *self,
-                             gboolean       visible)
+                              gboolean        visible)
 {
   g_assert (IDE_IS_EDITOR_FRAME (self));
 
@@ -216,7 +216,7 @@ ide_editor_frame_animate_map (IdeEditorFrame *self,
 
 static void
 ide_editor_frame_show_map (IdeEditorFrame *self,
-                          IdeSourceMap  *source_map)
+                           IdeSourceMap   *source_map)
 {
   g_assert (IDE_IS_EDITOR_FRAME (self));
   g_assert (IDE_IS_SOURCE_MAP (source_map));
@@ -226,7 +226,7 @@ ide_editor_frame_show_map (IdeEditorFrame *self,
 
 static void
 ide_editor_frame_hide_map (IdeEditorFrame *self,
-                          IdeSourceMap  *source_map)
+                           IdeSourceMap   *source_map)
 {
   g_assert (IDE_IS_EDITOR_FRAME (self));
   g_assert (IDE_IS_SOURCE_MAP (source_map));
@@ -240,7 +240,7 @@ ide_editor_frame_hide_map (IdeEditorFrame *self,
 
 static void
 ide_editor_frame_set_position_label (IdeEditorFrame *self,
-                                    const gchar   *text)
+                                     const gchar    *text)
 {
   g_return_if_fail (IDE_IS_EDITOR_FRAME (self));
 
@@ -314,8 +314,8 @@ ide_editor_frame_update_search_position_label (IdeEditorFrame *self)
 
 static void
 ide_editor_frame_on_search_occurrences_notify (IdeEditorFrame          *self,
-                                              GParamSpec             *pspec,
-                                              GtkSourceSearchContext *search_context)
+                                               GParamSpec              *pspec,
+                                               GtkSourceSearchContext  *search_context)
 {
   g_assert (IDE_IS_EDITOR_FRAME (self));
   g_assert (GTK_SOURCE_IS_SEARCH_CONTEXT (search_context));
@@ -710,13 +710,13 @@ ide_editor_frame_grab_focus (GtkWidget *widget)
 
 static void
 ide_editor_frame__drag_data_received (IdeEditorFrame    *self,
-                                     GdkDragContext   *context,
-                                     gint              x,
-                                     gint              y,
-                                     GtkSelectionData *selection_data,
-                                     guint             info,
-                                     guint             timestamp,
-                                     GtkWidget        *widget)
+                                      GdkDragContext    *context,
+                                      gint               x,
+                                      gint               y,
+                                      GtkSelectionData  *selection_data,
+                                      guint              info,
+                                      guint              timestamp,
+                                      GtkWidget         *widget)
 {
   gchar **uri_list;
 
@@ -759,8 +759,8 @@ ide_editor_frame__drag_data_received (IdeEditorFrame    *self,
 
 static gboolean
 ide_editor_frame__search_key_press_event (IdeEditorFrame *self,
-                                         GdkEventKey   *event,
-                                         GdTaggedEntry *entry)
+                                          GdkEventKey    *event,
+                                          GdTaggedEntry  *entry)
 {
   g_assert (IDE_IS_EDITOR_FRAME (self));
   g_assert (GD_IS_TAGGED_ENTRY (entry));
@@ -836,8 +836,8 @@ ide_editor_frame__replace_key_press_event (IdeEditorFrame *self,
 
 static gboolean
 ide_editor_frame__source_view_focus_in_event (IdeEditorFrame *self,
-                                             GdkEventKey   *event,
-                                             IdeSourceView *source_view)
+                                              GdkEventKey    *event,
+                                              IdeSourceView  *source_view)
 {
   GtkTextBuffer *buffer;
 
@@ -926,7 +926,7 @@ ide_editor_frame_set_show_map (IdeEditorFrame *self,
 
 static void
 ide_editor_frame_set_auto_hide_map (IdeEditorFrame *self,
-                                   gboolean       auto_hide_map)
+                                    gboolean        auto_hide_map)
 {
   g_assert (IDE_IS_EDITOR_FRAME (self));
 
@@ -939,35 +939,6 @@ ide_editor_frame_set_auto_hide_map (IdeEditorFrame *self,
     }
 }
 
-static void
-ide_editor_frame__source_view_populate_popup (IdeEditorFrame *self,
-                                             GtkWidget     *popup,
-                                             IdeSourceView *source_view)
-{
-  g_assert (IDE_IS_EDITOR_FRAME (self));
-  g_assert (GTK_IS_WIDGET (popup));
-  g_assert (IDE_IS_SOURCE_VIEW (source_view));
-
-  if (GTK_IS_MENU_SHELL (popup))
-    {
-      GtkWidget *item;
-      GtkWidget *sep;
-
-      sep = g_object_new (GTK_TYPE_SEPARATOR_MENU_ITEM,
-                          "visible", TRUE,
-                          NULL);
-      gtk_menu_shell_append (GTK_MENU_SHELL (popup), sep);
-
-      item = g_object_new (GTK_TYPE_MENU_ITEM,
-                           "action-name", "view.reveal",
-                           "label", _("Re_veal in Project Tree"),
-                           "use-underline", TRUE,
-                           "visible", TRUE,
-                           NULL);
-      gtk_menu_shell_append (GTK_MENU_SHELL (popup), item);
-    }
-}
-
 static void
 ide_editor_frame__search_populate_popup (IdeEditorFrame *self,
                                          GtkWidget      *popup,
@@ -1040,12 +1011,6 @@ ide_editor_frame_constructed (GObject *object)
                            self,
                            G_CONNECT_SWAPPED);
 
-  g_signal_connect_object (self->source_view,
-                           "populate-popup",
-                           G_CALLBACK (ide_editor_frame__source_view_populate_popup),
-                           self,
-                           G_CONNECT_SWAPPED);
-
   g_signal_connect_object (self->search_entry,
                            "key-press-event",
                            G_CALLBACK (ide_editor_frame__search_key_press_event),
@@ -1107,9 +1072,9 @@ ide_editor_frame_dispose (GObject *object)
 
 static void
 ide_editor_frame_get_property (GObject    *object,
-                              guint       prop_id,
-                              GValue     *value,
-                              GParamSpec *pspec)
+                               guint       prop_id,
+                               GValue     *value,
+                               GParamSpec *pspec)
 {
   IdeEditorFrame *self = IDE_EDITOR_FRAME (object);
 
@@ -1138,9 +1103,9 @@ ide_editor_frame_get_property (GObject    *object,
 
 static void
 ide_editor_frame_set_property (GObject      *object,
-                              guint         prop_id,
-                              const GValue *value,
-                              GParamSpec   *pspec)
+                               guint         prop_id,
+                               const GValue *value,
+                               GParamSpec   *pspec)
 {
   IdeEditorFrame *self = IDE_EDITOR_FRAME (object);
 
