@@ -20,6 +20,7 @@
 
 #include <egg-signal-group.h>
 #include <glib/gi18n.h>
+#include <pnl.h>
 
 #include "ide-context.h"
 #include "ide-debug.h"
@@ -27,20 +28,29 @@
 #include "buffers/ide-buffer-manager.h"
 #include "buffers/ide-buffer.h"
 #include "editor/ide-editor-perspective.h"
+#include "editor/ide-editor-spell-widget.h"
 #include "editor/ide-editor-view.h"
 #include "util/ide-gtk.h"
 #include "workbench/ide-layout-grid.h"
+#include "workbench/ide-layout-pane.h"
 #include "workbench/ide-workbench.h"
 #include "workbench/ide-workbench-header-bar.h"
 
+#define OVERLAY_REVEAL_DURATION 300
+
 struct _IdeEditorPerspective
 {
-  IdeLayout              parent_instance;
+  PnlDockOverlay         parent_instance;
 
+  GtkWidget             *active_view;
+  IdeLayout             *layout;
   IdeLayoutGrid         *grid;
   GSimpleActionGroup    *actions;
 
   EggSignalGroup        *buffer_manager_signals;
+
+  gint                   right_pane_position;
+  guint                  spellchecker_opened : 1;
 };
 
 typedef struct
@@ -56,9 +66,17 @@ static void ide_editor_perspective_focus_location_full (IdeEditorPerspective
                                                         IdeSourceLocation       *location,
                                                         gboolean                 open_if_not_found);
 
-G_DEFINE_TYPE_EXTENDED (IdeEditorPerspective, ide_editor_perspective, IDE_TYPE_LAYOUT, 0,
+G_DEFINE_TYPE_EXTENDED (IdeEditorPerspective, ide_editor_perspective, PNL_TYPE_DOCK_OVERLAY, 0,
                         G_IMPLEMENT_INTERFACE (IDE_TYPE_PERSPECTIVE, ide_perspective_iface_init))
 
+enum {
+  PROP_0,
+  PROP_ACTIVE_VIEW,
+  LAST_PROP
+};
+
+static GParamSpec *properties [LAST_PROP];
+
 enum {
   VIEW_ADDED,
   VIEW_REMOVED,
@@ -79,19 +97,19 @@ ide_editor_perspective_restore_panel_state (IdeEditorPerspective *self)
 
   settings = g_settings_new ("org.gnome.builder.workbench");
 
-  pane = pnl_dock_bin_get_left_edge (PNL_DOCK_BIN (self));
+  pane = pnl_dock_bin_get_left_edge (PNL_DOCK_BIN (self->layout));
   reveal = g_settings_get_boolean (settings, "left-visible");
   position = g_settings_get_int (settings, "left-position");
   pnl_dock_revealer_set_reveal_child (PNL_DOCK_REVEALER (pane), reveal);
   pnl_dock_revealer_set_position (PNL_DOCK_REVEALER (pane), position);
 
-  pane = pnl_dock_bin_get_right_edge (PNL_DOCK_BIN (self));
+  pane = pnl_dock_bin_get_right_edge (PNL_DOCK_BIN (self->layout));
   reveal = g_settings_get_boolean (settings, "right-visible");
   position = g_settings_get_int (settings, "right-position");
   pnl_dock_revealer_set_reveal_child (PNL_DOCK_REVEALER (pane), reveal);
   pnl_dock_revealer_set_position (PNL_DOCK_REVEALER (pane), position);
 
-  pane = pnl_dock_bin_get_bottom_edge (PNL_DOCK_BIN (self));
+  pane = pnl_dock_bin_get_bottom_edge (PNL_DOCK_BIN (self->layout));
   reveal = g_settings_get_boolean (settings, "bottom-visible");
   position = g_settings_get_int (settings, "bottom-position");
   pnl_dock_revealer_set_reveal_child (PNL_DOCK_REVEALER (pane), reveal);
@@ -110,19 +128,19 @@ ide_editor_perspective_save_panel_state (IdeEditorPerspective *self)
 
   settings = g_settings_new ("org.gnome.builder.workbench");
 
-  pane = pnl_dock_bin_get_left_edge (PNL_DOCK_BIN (self));
+  pane = pnl_dock_bin_get_left_edge (PNL_DOCK_BIN (self->layout));
   position = pnl_dock_revealer_get_position (PNL_DOCK_REVEALER (pane));
   reveal = pnl_dock_revealer_get_reveal_child (PNL_DOCK_REVEALER (pane));
   g_settings_set_boolean (settings, "left-visible", reveal);
   g_settings_set_int (settings, "left-position", position);
 
-  pane = pnl_dock_bin_get_right_edge (PNL_DOCK_BIN (self));
+  pane = pnl_dock_bin_get_right_edge (PNL_DOCK_BIN (self->layout));
   position = pnl_dock_revealer_get_position (PNL_DOCK_REVEALER (pane));
   reveal = pnl_dock_revealer_get_reveal_child (PNL_DOCK_REVEALER (pane));
   g_settings_set_boolean (settings, "right-visible", reveal);
   g_settings_set_int (settings, "right-position", position);
 
-  pane = pnl_dock_bin_get_bottom_edge (PNL_DOCK_BIN (self));
+  pane = pnl_dock_bin_get_bottom_edge (PNL_DOCK_BIN (self->layout));
   position = pnl_dock_revealer_get_position (PNL_DOCK_REVEALER (pane));
   reveal = pnl_dock_revealer_get_reveal_child (PNL_DOCK_REVEALER (pane));
   g_settings_set_boolean (settings, "bottom-visible", reveal);
@@ -357,6 +375,25 @@ ide_editor_perspective_grid_empty (IdeEditorPerspective *self,
     gtk_stack_set_visible_child_name (GTK_STACK (stack), "empty_state");
 }
 
+static void
+ide_editor_perspective_get_property (GObject    *object,
+                                     guint       prop_id,
+                                     GValue     *value,
+                                     GParamSpec *pspec)
+{
+  IdeEditorPerspective *self = IDE_EDITOR_PERSPECTIVE (object);
+
+  switch (prop_id)
+    {
+    case PROP_ACTIVE_VIEW:
+      g_value_set_object (value, ide_editor_perspective_get_active_view (self));
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
+    }
+}
+
 static void
 ide_editor_perspective_class_init (IdeEditorPerspectiveClass *klass)
 {
@@ -364,14 +401,25 @@ ide_editor_perspective_class_init (IdeEditorPerspectiveClass *klass)
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
   GtkContainerClass *container_class = GTK_CONTAINER_CLASS (klass);
 
+  object_class->get_property = ide_editor_perspective_get_property;
   object_class->finalize = ide_editor_perspective_finalize;
 
   container_class->add = ide_editor_perspective_add;
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/builder/ui/ide-editor-perspective.ui");
+  gtk_widget_class_bind_template_child (widget_class, IdeEditorPerspective, layout);
   gtk_widget_class_bind_template_child (widget_class, IdeEditorPerspective, actions);
   gtk_widget_class_bind_template_child (widget_class, IdeEditorPerspective, grid);
 
+  properties [PROP_ACTIVE_VIEW] =
+    g_param_spec_object ("active-view",
+                         "Active View",
+                         "Active View",
+                         GTK_TYPE_WIDGET,
+                         (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_properties (object_class, LAST_PROP, properties);
+
   signals[VIEW_ADDED] =
     g_signal_new ("view-added",
                   G_TYPE_FROM_CLASS (klass),
@@ -393,6 +441,18 @@ ide_editor_perspective_class_init (IdeEditorPerspectiveClass *klass)
                   GTK_TYPE_WIDGET);
 }
 
+static void
+ide_editor_perspective_active_view_notify_cb (IdeEditorPerspective *self,
+                                              GParamSpec           *pspec,
+                                              IdeLayout            *layout)
+{
+  g_assert (IDE_IS_EDITOR_PERSPECTIVE (self));
+  g_assert (IDE_IS_LAYOUT (layout));
+
+  self->active_view = ide_layout_get_active_view (layout);
+  g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_ACTIVE_VIEW]);
+}
+
 static void
 ide_editor_perspective_init (IdeEditorPerspective *self)
 {
@@ -434,7 +494,7 @@ ide_editor_perspective_init (IdeEditorPerspective *self)
   g_action_map_add_action_entries (G_ACTION_MAP (self->actions), entries,
                                    G_N_ELEMENTS (entries), self);
 
-  actions = gtk_widget_get_action_group (GTK_WIDGET (self), "dockbin");
+  actions = gtk_widget_get_action_group (GTK_WIDGET (self->layout), "dockbin");
 
   for (i = 0; proxy_actions[i]; i++)
     {
@@ -448,6 +508,13 @@ ide_editor_perspective_init (IdeEditorPerspective *self)
 
   ide_widget_set_context_handler (GTK_WIDGET (self),
                                   ide_editor_perspective_context_set);
+
+  g_signal_connect_swapped (self->layout,
+                            "notify::active-view",
+                            G_CALLBACK (ide_editor_perspective_active_view_notify_cb),
+                            self);
+
+  ide_editor_perspective_active_view_notify_cb (self, NULL, self->layout);
 }
 
 static gchar *
@@ -645,3 +712,289 @@ ide_editor_perspective_focus_location (IdeEditorPerspective *self,
 {
   ide_editor_perspective_focus_location_full (self, location, TRUE);
 }
+
+/**
+ * ide_editor_perspective_get_layout:
+ * @self: A #IdeEditorPerspective
+ *
+ * Gets the #IdeLayout widget for the editor perspective.
+ *
+ * Returns: (transfer none) (nullable): A #IdeLayout or %NULL.
+ */
+IdeLayout *
+ide_editor_perspective_get_layout (IdeEditorPerspective *self)
+{
+  g_return_val_if_fail (IDE_IS_EDITOR_PERSPECTIVE (self), NULL);
+
+  return self->layout;
+}
+
+/**
+ * ide_editor_perspective_get_active_view:
+ *
+ * Returns: (transfer none) (nullable): An #IdeLayoutView or %NULL.
+ */
+GtkWidget *
+ide_editor_perspective_get_active_view (IdeEditorPerspective *self)
+{
+
+  g_return_val_if_fail (IDE_IS_EDITOR_PERSPECTIVE (self), NULL);
+
+  return self->active_view;
+}
+
+/**
+ * ide_editor_perspective_get_center_widget:
+ * @self: A #IdeEditorPerspective
+ *
+ * Gets the center widget for the editor perspective.
+ *
+ * Returns: (transfer none) (nullable): A #GtkWidget or %NULL.
+ */
+GtkWidget *
+ide_editor_perspective_get_center_widget (IdeEditorPerspective *self)
+{
+  g_return_val_if_fail (IDE_IS_EDITOR_PERSPECTIVE (self), NULL);
+
+  return pnl_dock_bin_get_center_widget (PNL_DOCK_BIN (self->layout));
+}
+
+/**
+ * ide_editor_perspective_get_top_edge:
+ * Returns: (transfer none): A #GtkWidget
+ */
+GtkWidget *
+ide_editor_perspective_get_top_edge (IdeEditorPerspective *self)
+{
+  g_return_val_if_fail (IDE_IS_EDITOR_PERSPECTIVE (self), NULL);
+
+  return pnl_dock_bin_get_top_edge (PNL_DOCK_BIN (self->layout));
+}
+
+/**
+ * ide_editor_perspective_get_left_edge:
+ * Returns: (transfer none): A #GtkWidget
+ */
+GtkWidget *
+ide_editor_perspective_get_left_edge (IdeEditorPerspective *self)
+{
+  g_return_val_if_fail (IDE_IS_EDITOR_PERSPECTIVE (self), NULL);
+
+  return pnl_dock_bin_get_left_edge (PNL_DOCK_BIN (self->layout));
+}
+
+/**
+ * ide_editor_perspective_get_bottom_edge:
+ * Returns: (transfer none): A #GtkWidget
+ */
+GtkWidget *
+ide_editor_perspective_get_bottom_edge (IdeEditorPerspective *self)
+{
+  g_return_val_if_fail (IDE_IS_EDITOR_PERSPECTIVE (self), NULL);
+
+  return pnl_dock_bin_get_bottom_edge (PNL_DOCK_BIN (self->layout));
+}
+
+/**
+ * ide_editor_perspective_get_right_edge:
+ * Returns: (transfer none): A #GtkWidget
+ */
+GtkWidget *
+ide_editor_perspective_get_right_edge (IdeEditorPerspective *self)
+{
+  g_return_val_if_fail (IDE_IS_EDITOR_PERSPECTIVE (self), NULL);
+
+  return pnl_dock_bin_get_right_edge (PNL_DOCK_BIN (self->layout));
+}
+
+/**
+ * ide_editor_perspective_get_overlay_edge:
+ * self: an #IdeEditorPerspective.
+ * position: a #GtkPositionType.
+ *
+ * Returns: (transfer none): A #PnlDockOverlayEdge
+ */
+PnlDockOverlayEdge *
+ide_editor_perspective_get_overlay_edge (IdeEditorPerspective *self,
+                                         GtkPositionType       position)
+{
+  g_return_val_if_fail (IDE_IS_EDITOR_PERSPECTIVE (self), NULL);
+
+  return pnl_dock_overlay_get_edge (PNL_DOCK_OVERLAY (self), position);
+}
+
+static GtkOrientation
+get_orientation_from_position_type (GtkPositionType position_type)
+{
+  if (position_type == GTK_POS_LEFT || position_type == GTK_POS_RIGHT)
+    return GTK_ORIENTATION_HORIZONTAL;
+  else
+    return GTK_ORIENTATION_VERTICAL;
+}
+
+/* Triggered at the start of the animation */
+static void
+overlay_child_reveal_notify_cb (IdeEditorPerspective *self,
+                                GParamSpec           *pspec,
+                                PnlDockOverlayEdge   *edge)
+{
+  IdeLayoutPane *pane;
+  gboolean reveal;
+
+  g_assert (IDE_IS_EDITOR_PERSPECTIVE (self));
+  g_assert (PNL_IS_DOCK_OVERLAY_EDGE (edge));
+
+  gtk_container_child_get (GTK_CONTAINER (self), GTK_WIDGET (edge),
+                           "reveal", &reveal,
+                           NULL);
+
+  if (!reveal && self->spellchecker_opened)
+    {
+      g_signal_handlers_disconnect_by_func (edge,
+                                            overlay_child_reveal_notify_cb,
+                                            self);
+
+      pane = IDE_LAYOUT_PANE (pnl_dock_bin_get_right_edge (PNL_DOCK_BIN (self->layout)));
+      pnl_dock_revealer_animate_to_position (PNL_DOCK_REVEALER (pane),
+                                             self->right_pane_position,
+                                             OVERLAY_REVEAL_DURATION);
+    }
+}
+
+/* Triggered at the end of the animation */
+static void
+overlay_child_revealed_notify_cb (IdeEditorPerspective *self,
+                                  GParamSpec           *pspec,
+                                  PnlDockOverlayEdge   *edge)
+{
+  GtkWidget *child;
+  gboolean revealed;
+
+  g_assert (IDE_IS_EDITOR_PERSPECTIVE (self));
+  g_assert (PNL_IS_DOCK_OVERLAY_EDGE (edge));
+
+  gtk_container_child_get (GTK_CONTAINER (self), GTK_WIDGET (edge),
+                           "revealed", &revealed,
+                           NULL);
+
+  if (!revealed && self->spellchecker_opened)
+    {
+      g_signal_handlers_disconnect_by_func (edge,
+                                            overlay_child_revealed_notify_cb,
+                                            self);
+
+      child = gtk_bin_get_child (GTK_BIN (edge));
+      g_assert (child != NULL);
+      gtk_container_remove (GTK_CONTAINER (edge), child);
+      self->spellchecker_opened = FALSE;
+    }
+  else if (revealed)
+    self->spellchecker_opened = TRUE;
+}
+
+static void
+show_spell_checker (IdeEditorPerspective *self,
+                    PnlDockOverlayEdge   *overlay_edge,
+                    IdeLayoutPane        *pane)
+{
+  GtkOrientation pane_orientation;
+  GtkPositionType pane_position_type;
+  GtkOrientation overlay_orientation;
+  GtkPositionType overlay_position_type;
+  gint overlay_size;
+
+  g_assert (IDE_IS_EDITOR_PERSPECTIVE (self));
+  g_assert (gtk_bin_get_child (GTK_BIN (overlay_edge)) != NULL);
+
+  pane_position_type = pnl_dock_bin_edge_get_edge (PNL_DOCK_BIN_EDGE (pane));
+  overlay_position_type = pnl_dock_overlay_edge_get_edge (overlay_edge);
+
+  pane_orientation = get_orientation_from_position_type (pane_position_type);
+  overlay_orientation = get_orientation_from_position_type (overlay_position_type);
+
+  g_assert (pane_orientation == overlay_orientation);
+
+  if (pnl_dock_revealer_get_position_set (PNL_DOCK_REVEALER (pane)))
+    self->right_pane_position = pnl_dock_revealer_get_position (PNL_DOCK_REVEALER (pane));
+  else
+    {
+      if (overlay_orientation == GTK_ORIENTATION_HORIZONTAL)
+        gtk_widget_get_preferred_width (GTK_WIDGET (pane), NULL, &self->right_pane_position);
+      else
+        gtk_widget_get_preferred_height (GTK_WIDGET (pane), NULL, &self->right_pane_position);
+    }
+
+  if (overlay_orientation == GTK_ORIENTATION_HORIZONTAL)
+    gtk_widget_get_preferred_width (GTK_WIDGET (overlay_edge), NULL, &overlay_size);
+  else
+    gtk_widget_get_preferred_height (GTK_WIDGET (overlay_edge), NULL, &overlay_size);
+
+  g_signal_connect_object (overlay_edge,
+                           "child-notify::reveal",
+                           G_CALLBACK (overlay_child_reveal_notify_cb),
+                           self,
+                           G_CONNECT_SWAPPED);
+  g_signal_connect_object (overlay_edge,
+                           "child-notify::revealed",
+                           G_CALLBACK (overlay_child_revealed_notify_cb),
+                           self,
+                           G_CONNECT_SWAPPED);
+
+  pnl_dock_revealer_animate_to_position (PNL_DOCK_REVEALER (pane),
+                                         overlay_size,
+                                         OVERLAY_REVEAL_DURATION);
+  gtk_container_child_set (GTK_CONTAINER (self), GTK_WIDGET (overlay_edge),
+                           "reveal", TRUE,
+                           NULL);
+}
+
+static GtkWidget *
+create_spellchecker_widget (IdeSourceView *source_view)
+{
+  GtkWidget *spellchecker_widget;
+  GtkWidget *scroll_window;
+  GtkWidget *spell_widget;
+
+  g_assert (IDE_IS_SOURCE_VIEW (source_view));
+
+  spellchecker_widget = g_object_new (GTK_TYPE_BOX,
+                                      "visible", TRUE,
+                                      "expand", TRUE,
+                                      NULL);
+  scroll_window = g_object_new (GTK_TYPE_SCROLLED_WINDOW,
+                                "visible", TRUE,
+                                "expand", TRUE,
+                                "propagate-natural-width", TRUE,
+                                NULL);
+  spell_widget = ide_editor_spell_widget_new (source_view);
+  gtk_box_pack_start (GTK_BOX (spellchecker_widget), scroll_window, TRUE, TRUE, 0);
+  gtk_container_add (GTK_CONTAINER (scroll_window), spell_widget);
+  gtk_widget_show_all (spellchecker_widget);
+
+  return spellchecker_widget;
+}
+
+void
+ide_editor_perspective_show_spellchecker (IdeEditorPerspective *self,
+                                          IdeSourceView        *source_view)
+{
+  GtkWidget *spellchecker_widget;
+  PnlDockOverlayEdge *overlay_edge;
+  IdeLayoutPane *pane;
+
+  g_return_if_fail (IDE_IS_EDITOR_PERSPECTIVE (self));
+  g_return_if_fail (IDE_IS_SOURCE_VIEW (source_view));
+
+  if (!self->spellchecker_opened)
+    {
+      self->spellchecker_opened = TRUE;
+      spellchecker_widget = create_spellchecker_widget (source_view);
+
+      pnl_overlay_add_child (PNL_DOCK_OVERLAY (self), spellchecker_widget, "right");
+      overlay_edge = ide_editor_perspective_get_overlay_edge (self, GTK_POS_RIGHT);
+      gtk_widget_set_child_visible (GTK_WIDGET (overlay_edge), TRUE);
+
+      pane = IDE_LAYOUT_PANE (pnl_dock_bin_get_right_edge (PNL_DOCK_BIN (self->layout)));
+      show_spell_checker (self, overlay_edge, pane);
+    }
+}
