@@ -655,6 +655,8 @@ static void
 ide_greeter_perspective_open_clicked (IdeGreeterPerspective *self,
                                       GtkButton             *open_button)
 {
+  g_autoptr(GSettings) settings = NULL;
+  g_autofree gchar *projects_dir = NULL;
   GtkFileChooserDialog *dialog;
   GtkWidget *toplevel;
   PeasEngine *engine;
@@ -764,6 +766,10 @@ ide_greeter_perspective_open_clicked (IdeGreeterPerspective *self,
 
   gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dialog), all_filter);
 
+  settings = g_settings_new ("org.gnome.builder");
+  projects_dir = g_settings_get_string (settings, "projects-directory");
+  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), projects_dir);
+
   gtk_window_present (GTK_WINDOW (dialog));
 }
 
