@@ -19,10 +19,8 @@
 #ifndef IDE_C_FORMAT_PROVIDER_H
 #define IDE_C_FORMAT_PROVIDER_H
 
-#include <gtksourceview/gtksourcecompletionprovider.h>
-
-#include "ide-completion-provider.h"
-#include "ide-object.h"
+#include <gtksourceview/gtksource.h>
+#include <ide.h>
 
 G_BEGIN_DECLS
 
