@@ -95,13 +95,36 @@ ide_git_clone_widget_uri_changed (IdeGitCloneWidget *self,
                                   GtkEntry          *entry)
 {
   g_autoptr(IdeVcsUri) uri = NULL;
+  g_autoptr(GString) str = NULL;
   const gchar *text;
   gboolean is_ready = FALSE;
+  gboolean matches = TRUE;
 
   g_assert (IDE_IS_GIT_CLONE_WIDGET (self));
   g_assert (GTK_IS_ENTRY (entry));
 
   text = gtk_entry_get_text (entry);
+  str = g_string_new (NULL);
+
+  for (const gchar *ptr = text; *ptr; ptr = g_utf8_next_char (ptr))
+    {
+      gunichar ch = g_utf8_get_char (ptr);
+
+      if (!g_unichar_isspace (ch))
+        {
+          g_string_append_unichar (str, ch);
+          matches = FALSE;
+        }
+    }
+
+  if (!matches)
+    {
+      g_signal_handlers_block_by_func (entry, G_CALLBACK (ide_git_clone_widget_uri_changed), self);
+      text = str->str;
+      gtk_entry_set_text (entry, text);
+      g_signal_handlers_unblock_by_func (entry, G_CALLBACK (ide_git_clone_widget_uri_changed), self);
+    }
+
   uri = ide_vcs_uri_new (text);
 
   if (uri != NULL)
@@ -326,6 +349,11 @@ ide_git_clone_widget_worker (GTask        *task,
   callbacks = g_object_new (IDE_TYPE_GIT_REMOTE_CALLBACKS, NULL);
   progress = ide_git_remote_callbacks_get_progress (IDE_GIT_REMOTE_CALLBACKS (callbacks));
   g_object_bind_property (progress, "fraction", self->clone_progress, "fraction", 0);
+  g_signal_connect_object (cancellable,
+                           "cancelled",
+                           G_CALLBACK (ide_git_remote_callbacks_cancel),
+                           callbacks,
+                           G_CONNECT_SWAPPED);
 
   fetch_options = ggit_fetch_options_new ();
   ggit_fetch_options_set_remote_callbacks (fetch_options, callbacks);
@@ -349,6 +377,9 @@ ide_git_clone_widget_worker (GTask        *task,
       return;
     }
 
+  if (g_task_return_error_if_cancelled (task))
+    return;
+
   req->project_file = ggit_repository_get_workdir (repository);
   g_timeout_add (0, finish_animation_in_idle, g_object_ref (task));
 
@@ -364,17 +395,28 @@ ide_git_clone_widget_clone_async (IdeGitCloneWidget   *self,
   g_autoptr(GTask) task = NULL;
   g_autoptr(GFile) location = NULL;
   g_autoptr(IdeVcsUri) uri = NULL;
+  g_autofree gchar *uristr = NULL;
   CloneRequest *req;
-  const gchar *uristr;
 
   g_return_if_fail (IDE_IS_GIT_CLONE_WIDGET (self));
   g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
 
   task = g_task_new (self, cancellable, callback, user_data);
+  g_task_set_source_tag (task, ide_git_clone_widget_clone_async);
+
+  /*
+   * ggit_repository_clone() will block and we don't have a good way to
+   * cancel it. So we need to return immediately (even though the clone
+   * will continue in the background for now).
+   *
+   * FIXME: Find Ggit API to cancel clone. We might need access to the
+   *    GgitRemote so we can ggit_remote_disconnect().
+   */
+  g_task_set_return_on_cancel (task, TRUE);
 
   gtk_label_set_label (self->clone_error_label, NULL);
 
-  uristr = gtk_entry_get_text (self->clone_uri_entry);
+  uristr = g_strstrip (g_strdup (gtk_entry_get_text (self->clone_uri_entry)));
   location = egg_file_chooser_entry_get_file (EGG_FILE_CHOOSER_ENTRY (self->clone_location_entry));
 
   uri = ide_vcs_uri_new (uristr);
@@ -404,6 +446,9 @@ ide_git_clone_widget_clone_async (IdeGitCloneWidget   *self,
   gtk_widget_set_sensitive (GTK_WIDGET (self->clone_location_entry), FALSE);
   gtk_widget_set_sensitive (GTK_WIDGET (self->clone_uri_entry), FALSE);
 
+  gtk_progress_bar_set_fraction (self->clone_progress, 0.0);
+  gtk_widget_show (GTK_WIDGET (self->clone_progress));
+
   g_task_set_task_data (task, req, clone_request_free);
   g_task_run_in_thread (task, ide_git_clone_widget_worker);
 }
@@ -413,13 +458,24 @@ ide_git_clone_widget_clone_finish (IdeGitCloneWidget  *self,
                                    GAsyncResult       *result,
                                    GError            **error)
 {
+  GError *local_error = NULL;
+  gboolean ret;
+
   g_return_val_if_fail (IDE_IS_GIT_CLONE_WIDGET (self), FALSE);
   g_return_val_if_fail (G_IS_TASK (result), FALSE);
 
+  ret = g_task_propagate_boolean (G_TASK (result), &local_error);
+
+  /* Only hide progress if we were cancelled */
+  if (g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+    gtk_widget_hide (GTK_WIDGET (self->clone_progress));
+
   gtk_spinner_stop (self->clone_spinner);
 
   gtk_widget_set_sensitive (GTK_WIDGET (self->clone_location_entry), TRUE);
   gtk_widget_set_sensitive (GTK_WIDGET (self->clone_uri_entry), TRUE);
 
-  return g_task_propagate_boolean (G_TASK (result), error);
+  g_propagate_error (error, local_error);
+
+  return ret;
 }
