@@ -95,17 +95,17 @@ gb_terminal_view_discover_shell (GbTerminalView  *self,
   if (cached_shell != NULL)
     return g_strdup (cached_shell);
 
-  command = g_strdup_printf ("sh -c 'getent passwd | grep ^%s: | cut -f 7 -d :'",
+  command = g_strdup_printf ("sh -c 'getent passwd | grep ^%s: | head -n1 | cut -f 7 -d :'",
                              g_get_user_name ());
 
   if (!g_shell_parse_argv (command, NULL, &argv, error))
     return NULL;
 
-  if (self->runtime != NULL)
-    launcher = ide_runtime_create_launcher (self->runtime, NULL);
-
-  if (launcher == NULL)
-    launcher = ide_subprocess_launcher_new (G_SUBPROCESS_FLAGS_STDOUT_PIPE);
+  /*
+   * We don't use the runtime shell here, because we want to know
+   * what the host thinks the user shell should be.
+   */
+  launcher = ide_subprocess_launcher_new (G_SUBPROCESS_FLAGS_STDOUT_PIPE);
 
   ide_subprocess_launcher_set_run_on_host (launcher, TRUE);
   ide_subprocess_launcher_set_clear_env (launcher, FALSE);
@@ -191,7 +191,6 @@ static void
 gb_terminal_respawn (GbTerminalView *self,
                      VteTerminal    *terminal)
 {
-  g_autoptr(GPtrArray) args = NULL;
   g_autoptr(IdeSubprocess) subprocess = NULL;
   g_autoptr(IdeSubprocessLauncher) launcher = NULL;
   g_autofree gchar *workpath = NULL;
@@ -243,10 +242,6 @@ gb_terminal_respawn (GbTerminalView *self,
       g_clear_error (&error);
     }
 
-  args = g_ptr_array_new ();
-  g_ptr_array_add (args, (gchar *)shell);
-  g_ptr_array_add (args, NULL);
-
   pty = vte_terminal_pty_new_sync (terminal,
                                    VTE_PTY_DEFAULT | VTE_PTY_NO_LASTLOG | VTE_PTY_NO_UTMP | VTE_PTY_NO_WTMP,
                                    NULL,
@@ -264,7 +259,15 @@ gb_terminal_respawn (GbTerminalView *self,
     IDE_GOTO (failure);
 
   if (self->runtime != NULL)
-    launcher = ide_runtime_create_launcher (self->runtime, NULL);
+    {
+      launcher = ide_runtime_create_launcher (self->runtime, NULL);
+
+      if (!ide_runtime_contains_program_in_path (self->runtime, shell, NULL))
+        {
+          g_free (shell);
+          shell = g_strdup ("/bin/bash");
+        }
+    }
 
   if (launcher == NULL)
     launcher = ide_subprocess_launcher_new (0);
@@ -273,7 +276,7 @@ gb_terminal_respawn (GbTerminalView *self,
   ide_subprocess_launcher_set_run_on_host (launcher, TRUE);
   ide_subprocess_launcher_set_clear_env (launcher, FALSE);
   ide_subprocess_launcher_set_cwd (launcher, workpath);
-  ide_subprocess_launcher_push_args (launcher, (const gchar * const *)args->pdata);
+  ide_subprocess_launcher_push_argv (launcher, shell);
   ide_subprocess_launcher_take_stdin_fd (launcher, tty_fd);
   ide_subprocess_launcher_take_stdout_fd (launcher, stdout_fd);
   ide_subprocess_launcher_take_stderr_fd (launcher, stderr_fd);
