@@ -1,8 +1,9 @@
 #include <fuzzy.h>
-#include <ide-line-reader.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include "util/ide-line-reader.h"
+
 int
 main (int argc,
       char *argv[])
